
/* Network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Big numbers - "arbitrary precision" arithmetic using strings */

#ifndef __BIGNUM_H
#define __BIGNUM_H

#include "darkstat.h"



#ifndef HAVE_QUAD

#define TWOPOW32 "4294967296"

typedef struct {
	dword hi, lo;
} i64;



inline dword num_from_char(const byte c);

/* note: output needs to be deallocated */
inline char *bignum_from_int(const dword n);

/* note: output needs to be deallocated */
inline char *add_bignum(char *a, char *b);

/* note: output needs to be deallocated */
inline char *mul_bignum(char *a, char *b);

inline void _add_64bit(i64 *dest, const dword src);

/* note: output needs to be deallocated */
inline char *bignum_from_64bit(const i64 val);

inline double db_div_64_d(const i64 num, const dword denom);
inline int i_div_64_db(const i64 num, const double denom);

inline int smaller_64(const i64 a, const i64 b);
inline int bigger_64(const i64 a, const i64 b);
inline i64 max_64(const i64 a, const i64 b);
inline i64 min_64(const i64 a, const i64 b);

inline char *add_commas(char *str, int want_free);

#define SET64(x,h,l) { x.hi = h;   x.lo = l; }
#define add_64bit(a,b) _add_64bit(&(a), b)
#define HI(x) x.hi
#define LO(x) x.lo



#else /* HAVE_QUAD */

typedef long long int i64;
#define HI(x) ((dword) (((x) >> 32) & 0xFFFFFFFF))
#define LO(x) ((dword) ((x) & 0xFFFFFFFF))
#define SET64(x,h,l) x = ( ( ((i64)(h)) & 0xFFFFFFFF ) << 32 ) |\
	( ((i64)(l)) & 0xFFFFFFFF )

#define smaller_64(a,b) ((a) < (b))
#define bigger_64(a,b) ((a) > (b))

#define max_64(a,b) ( ((a)>(b)) ? (a) : (b) )
#define min_64(a,b) ( ((a)<(b)) ? (a) : (b) )

#define db_div_64_d(q,d) ((double)(q) / (double)(d))
#define i_div_64_db(q,d) ( (int)((double)(q) / (double)(d)) )

#define add_64bit(a, b) a += (i64)(b)

#endif /* !HAVE_QUAD */



#endif

