
/* Network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* this is the DNS resolver */

#include "dns.h"
#include "host_db.h"
#include <netdb.h>
#include <netinet/in.h>
#include <assert.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/socket.h>

#define WAIT_BETWEEN_QUERIES 1 /* sec */
#define WAIT_BETWEEN_LOOPS 10 /* secs */

pthread_t dns_thread;
pthread_mutex_t db_mutex;
dns_status want_resolve = ON;



inline char *name_from_ip(dword ip_addr)
{
	struct hostent *hp;
	unsigned int ip;

	ip = htonl(ip_addr);
	hp = gethostbyaddr((char*)&ip, 4, AF_INET);

	if (hp)
		return strdup(hp->h_name);
	else
		return NULL;
}



inline void dns_sleep(int secs)
{
	int i;

	for (i=0; i<secs && !shutting_down; i++)
		usleep(1000*MSEC);
}



/* argument is ignored */
void dns_main(void *ptr)
{
	int i = 0;
	
	printf("DNS: Thread is awake.\n");
	fflush(stdout);

	while (host_db.used < 1 && !shutting_down)
	{
		if (verbose)
			printf("DNS: Waiting for the host_db to fill up.\n");

		fflush(stdout);
		dns_sleep(WAIT_BETWEEN_QUERIES);
	}

	while (!shutting_down)
	{
		host_record *h;
		int db_used;

		while (want_resolve == OFF)
		{
			if (verbose) printf("DNS: Disabled.\n");
			dns_sleep(WAIT_BETWEEN_LOOPS);
			if (shutting_down) pthread_exit(0);
		}

		pthread_mutex_lock(&db_mutex);
		h = (host_record*)db_get(host_db, i);
		db_used = host_db.used;
		pthread_mutex_unlock(&db_mutex);

		if (h->hostname == NULL)
		{
			h->hostname = name_from_ip(h->ip_addr);
			if (verbose)
			{
				if (h->hostname)
					printf("DNS: Resolved %s",
						h->hostname);
				else
					printf("DNS: Resolution failed");
				printf(" (%d/%d)\n", i+1, db_used);
			}

			if (want_resolve != IN_PROGRESS)
				dns_sleep(WAIT_BETWEEN_QUERIES);
		}

		i++;
		if (i == db_used)
		{
			i = 0;
			if (verbose)
				printf("DNS: Hit end of list -> sleeptime\n");
			if (want_resolve == IN_PROGRESS)
				want_resolve = OFF;
			dns_sleep(WAIT_BETWEEN_LOOPS);
		}
	}

	printf("DNS: Thread down.\n");
	pthread_exit(0);
}

